<?php
// Datei:       class.session.php
// Bentigt:    mind. 4.0.1pl2

/**
*   "Manueller" Session-Fallback mit PHP4
*
*   @author     Daniel T. Gorski <daniel.gorski@bluemars.de>
*
*	Gendert von: Falk Amthor (http://php.faprojects.de)
*/

class Session {
    var $version     = 106;     // V1.06
    var $usesCookies = false;   // Client nimmt Cookies an
    var $transSID    = false;   // Wurde mit --enable-trans-sid
                                // kompiliert  

//---------------------------------------------------------

/**
*   Konstruktor - nimmt, wenn gewnscht einen neuen
*   Session-Namen entgegen
*/    
    function Session($sessionName="session",$cookies=false) {
        global  $HTTP_POST_VARS,$HTTP_GET_VARS,
                $HTTP_COOKIE_VARS,$QUERY_STRING,$PHP_SELF;

        $this->sendNoCacheHeader();
		
        //  Session-Namen setzen, Session initialisieren   
        session_name(isset($sessionName)
            ? $sessionName
            : session_name());

        @session_start();
        
        //  Prfen ob die Session-ID die Standardlnge
        //  von 32 Zeichen hat,
        //  ansonsten Session-ID neu setzen 
        if (strlen(session_id()) != 32)
            {
                mt_srand ((double)microtime()*1000000);
                session_id(md5(uniqid(mt_rand())));
            }
        
        //  Prfen, ob eine Session-ID bergeben wurde
        //  (ber Cookie, POST oder GET)
        $IDpassed = false;

        if  (   isset($HTTP_COOKIE_VARS     [session_name()]) &&
                @strlen($HTTP_COOKIE_VARS   [session_name()]) == 32
            )   $IDpassed = true;

        if  (   isset($HTTP_POST_VARS       [session_name()]) &&
                @strlen($HTTP_POST_VARS     [session_name()]) == 32
            )   $IDpassed = true;

        if  (   isset($HTTP_GET_VARS        [session_name()]) &&
                @strlen($HTTP_GET_VARS      [session_name()]) == 32
            )   $IDpassed = true;
        
        if  (!$IDpassed)  
            {   
                // Es wurde keine (gltige) Session-ID bergeben.
                // Script-Parameter der URL zufgen
                
                $query = @$QUERY_STRING != "" ? "?".$QUERY_STRING : "";
             
               ##header("Status: 302 Found");  //nderung by FA
                
                // Script terminiert
                $this->redirectTo($PHP_SELF.$query);
            }
            
        // Wenn die Session-ID bergeben wurde, mu sie
        // nicht unbedingt gltig sein!
        
		// Wenn Cookies benutzt werden sollen
		if ($cookies)
		{		
		        // Fr weiteren Gebrauch merken    
		        $this->usesCookies =
		                       (isset($HTTP_COOKIE_VARS[session_name()]) &&
		                        @strlen($HTTP_COOKIE_VARS[session_name()])
		                        == 32);
		}
    }    
 
### -------------------------------------------------------
/**
*   Cacheing unterbinden
*
*   Ergnze/Override "session.cache_limiter = nocache"
*
*   @param  void
*   @return void
*/    
    function sendNoCacheHeader()    {        
        header("Expires: Sat, 05 Aug 2000 22:27:00 GMT");
        header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT");
        header("Cache-Control: no-cache, must-revalidate");
        header("Pragma: no-cache");
        header("Cache-Control: post-check=0, pre-check=0");   
		header ("Cache-Control: private");  //ergnzung von FA
    }

### -------------------------------------------------------
/**
*   HTTP-Redirect ausfhren (header("Location: ...")
*
*   @param  string  Ziel-Datei (z.B. "index.php")
*   @return void
*/   
    function redirectTo($url) {
        $url=$this->removeTrail($url);
		
		if (!$this->usesCookies || !$this->transSID)
		{
			$sid=session_name()."=".session_id();
			
			$i=split("\?",$url);
			if($i[1]){
				$sid="&".$sid;
			}else{
				$sid="?".$sid;
			}
			$o_url=$url;
			$a=split("\#",$url);
			$url=$a[0].$sid.$a[1];
			
		}
		header ("Request-URI: $o_url"); 
		header ("Content-Location: $o_url"); 
		header ("Location: $url");         
        exit;
    }

### -------------------------------------------------------
/**
*   Entfernt mgliche abschlieende "&" und "?"
*
*   @param  string  String
*   @return string  String ohne abschlieende "&" und "?"
*/
    function removeTrail($pathInfo) {
        $dummy = preg_match("/(.*)(?<!&|\?)/",$pathInfo,$match);
        return $match[0]; 
    }

### -------------------------------------------------------
/**
*   Fallback via GET - wenn Cookies ausgeschaltet sind
*
*   @param  string  Ziel-Datei
*   @return string  Ziel-Datei mit - bei Bedarf - angehngter Session-ID
*/
    function url($pathInfo)  {        
        if ($this->usesCookies || $this->transSID) return $pathInfo;

        // Anchor-Fragment extrahieren
        $dummyArray = split("#",$pathInfo);
        $pathInfo = $dummyArray[0];

        // evtl. (kaputte) Session-ID(s) aus dem Querystring entfernen
        $pathInfo = preg_replace(   "/[?|&]".session_name()."=[^&]*/",
                                    "",
                                    $pathInfo);
        
        // evtl. Query-Delimiter korrigieren
        if (preg_match("/&/",$pathInfo) && !preg_match("/\?/",$pathInfo))
            {
                // 4ter Parameter fr "preg_replace()" erst ab 4.0.1pl2
                $pathInfo = preg_replace("/&/","?",$pathInfo,1); 
            }
        
        // Restmll entsorgen
        $pathInfo = $this->removeTrail($pathInfo);
        
        // Session-Name und Session-ID frisch hinzufgen  
        $pathInfo .= preg_match("/\?/",$pathInfo) ? "&" : "?";
        $pathInfo .= session_name()."=".session_id();
        
        // Anchor-Fragment wieder anfgen
        $pathInfo .= isset($dummyArray[1]) ? "#".$dummyArray[1] : "";
        
        return $pathInfo;                       
    }
    
### -------------------------------------------------------
/**
*   Fallback via HIDDEN FIELD - wenn Cookies ausgeschaltet sind
*
*   Ohne Cookies erfolgt Fallback via HTML-Hidden-Field
*   (fr Formulare)
*   
*   @param  void
*   @return string  HTML-Hidden-Input-Tag mit der Session-ID
*/
    function hidden() {
        if ($this->usesCookies || $this->transSID) return "";
        return "<INPUT  type=\"hidden\"
                        name=\"".session_name()."\"
                        value=\"".session_id()."\">";
    }
} // of class    
    
?>